@@ -1,6 +1,6 @@
 var searchData=
 [
   ['absolute_2daxes_2edox',['absolute-axes.dox',['../absolute-axes_8dox.html',1,'']]],
-  ['absolute_20axes',['Absolute axes',['../absolute_axes.html',1,'']]],
+  ['absolute_20axes',['Absolute axes',['../absolute_axes.html',1,'touchscreens']]],
   ['accessing_20and_20destruction_20of_20events',['Accessing and destruction of events',['../group__event.html',1,'']]]
 ];
