@@ -13,6 +13,10 @@ var searchData=
   ['libinput_5fdevice_5fconfig_5fclick_5fget_5fmethod',['libinput_device_config_click_get_method',['../group__config.html#ga6aa5fc908b050beb6c0346a09478cad4',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fclick_5fget_5fmethods',['libinput_device_config_click_get_methods',['../group__config.html#ga30f433d7c21fc8e65dd218079ce1a2a9',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fclick_5fset_5fmethod',['libinput_device_config_click_set_method',['../group__config.html#gadfd0d6c30f8d87ae570c9159953de3c9',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fdwt_5fget_5fdefault_5fenabled',['libinput_device_config_dwt_get_default_enabled',['../group__config.html#ga4b386261e05d2a89973148e9ce56731e',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fdwt_5fget_5fenabled',['libinput_device_config_dwt_get_enabled',['../group__config.html#gaf350c6505b5736ec410878b5daf28142',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fdwt_5fis_5favailable',['libinput_device_config_dwt_is_available',['../group__config.html#ga0d440dd9a15b10784d3a72fb8086fd32',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fdwt_5fset_5fenabled',['libinput_device_config_dwt_set_enabled',['../group__config.html#ga5e30865097b8fa273c3e3cbcdff51b0f',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fget',['libinput_device_config_left_handed_get',['../group__config.html#ga998945b1c4f96a9abd3c3a26e5c4166c',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fget_5fdefault',['libinput_device_config_left_handed_get_default',['../group__config.html#gaa438a14e32894791e435cb43826ef3a9',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fis_5favailable',['libinput_device_config_left_handed_is_available',['../group__config.html#ga755352f2bbb869446aa591dd9d9c5eaf',1,'libinput.h']]],
@@ -69,9 +73,21 @@ var searchData=
   ['libinput_5fdispatch',['libinput_dispatch',['../group__base.html#ga271f910ed17461830e48d4cd20483a00',1,'libinput.h']]],
   ['libinput_5fevent_5fdestroy',['libinput_event_destroy',['../group__event.html#ga1df956c16e26cbbd911e553ec081022a',1,'libinput.h']]],
   ['libinput_5fevent_5fdevice_5fnotify_5fget_5fbase_5fevent',['libinput_event_device_notify_get_base_event',['../group__event.html#ga19d329b7685c3a994718cec3845db1a0',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fangle_5fdelta',['libinput_event_gesture_get_angle_delta',['../group__event__gesture.html#ga53ed099f051922a765f8e614d96a0fea',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fbase_5fevent',['libinput_event_gesture_get_base_event',['../group__event__gesture.html#gab46fd1f8b4d5e12bc4d365e3599dfc94',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fcancelled',['libinput_event_gesture_get_cancelled',['../group__event__gesture.html#ga797bdfac0367cea193fac960258bbcb8',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fdx',['libinput_event_gesture_get_dx',['../group__event__gesture.html#ga3888052854155ad133fa837e4f28d771',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fdx_5funaccelerated',['libinput_event_gesture_get_dx_unaccelerated',['../group__event__gesture.html#ga9220d6e748fada925665cf9075e3812a',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fdy',['libinput_event_gesture_get_dy',['../group__event__gesture.html#gabcaa25a116f982acce8d98c0290ef6a2',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fdy_5funaccelerated',['libinput_event_gesture_get_dy_unaccelerated',['../group__event__gesture.html#ga52acb898a13ca5e638cd77cdf5983362',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5ffinger_5fcount',['libinput_event_gesture_get_finger_count',['../group__event__gesture.html#gacc6909a8adb6992da9cf2638d332e009',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5fscale',['libinput_event_gesture_get_scale',['../group__event__gesture.html#ga946ad3a4d60e728e89e13a6066b59a20',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5ftime',['libinput_event_gesture_get_time',['../group__event__gesture.html#ga6cdd8a884a49e323d9691bde058c5e4e',1,'libinput.h']]],
+  ['libinput_5fevent_5fgesture_5fget_5ftime_5fusec',['libinput_event_gesture_get_time_usec',['../group__event__gesture.html#ga0c281ad149d3f5e90e96c3073d82765a',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fcontext',['libinput_event_get_context',['../group__event.html#ga28557ae0295f61c45f9b6e4ff7f86632',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fdevice',['libinput_event_get_device',['../group__event.html#ga8a49a089181960f7536344f7637ea777',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fdevice_5fnotify_5fevent',['libinput_event_get_device_notify_event',['../group__event.html#ga2bbc1391a3397d57d37ae3e2f8175abc',1,'libinput.h']]],
+  ['libinput_5fevent_5fget_5fgesture_5fevent',['libinput_event_get_gesture_event',['../group__event.html#gae173f30b70c265036bbdc9557f3115c4',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fkeyboard_5fevent',['libinput_event_get_keyboard_event',['../group__event.html#ga8378f8fdd01e8afbcea8ddbb5411570d',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fpointer_5fevent',['libinput_event_get_pointer_event',['../group__event.html#ga52acdc3afcf75efeb8153f50eef20615',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5ftouch_5fevent',['libinput_event_get_touch_event',['../group__event.html#gad2cba8252024bdcc341499bedd3d0be7',1,'libinput.h']]],
@@ -81,6 +97,7 @@ var searchData=
   ['libinput_5fevent_5fkeyboard_5fget_5fkey_5fstate',['libinput_event_keyboard_get_key_state',['../group__event__keyboard.html#gaf7b018edbc264ba81a2d475d42ebbd98',1,'libinput.h']]],
   ['libinput_5fevent_5fkeyboard_5fget_5fseat_5fkey_5fcount',['libinput_event_keyboard_get_seat_key_count',['../group__event__keyboard.html#ga75a647f08ef9df9db7a6ebb9b297add4',1,'libinput.h']]],
   ['libinput_5fevent_5fkeyboard_5fget_5ftime',['libinput_event_keyboard_get_time',['../group__event__keyboard.html#ga0f1a7bb1016816a277b52705d6ef2d71',1,'libinput.h']]],
+  ['libinput_5fevent_5fkeyboard_5fget_5ftime_5fusec',['libinput_event_keyboard_get_time_usec',['../group__event__keyboard.html#ga1403b8718219a759fdda0c3944fc9beb',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fabsolute_5fx',['libinput_event_pointer_get_absolute_x',['../group__event__pointer.html#gac221536b90c7beb70b70b52e710291b5',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fabsolute_5fx_5ftransformed',['libinput_event_pointer_get_absolute_x_transformed',['../group__event__pointer.html#ga26fa3d0b2bcc7f0a7939b84dc8207021',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fabsolute_5fy',['libinput_event_pointer_get_absolute_y',['../group__event__pointer.html#ga3e097a9bc6dbbb355fb1982912f72d4c',1,'libinput.h']]],
@@ -97,11 +114,13 @@ var searchData=
   ['libinput_5fevent_5fpointer_5fget_5fdy_5funaccelerated',['libinput_event_pointer_get_dy_unaccelerated',['../group__event__pointer.html#ga2a9d66358432b4bee5f8c201f8dc8906',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fseat_5fbutton_5fcount',['libinput_event_pointer_get_seat_button_count',['../group__event__pointer.html#ga1990b268bdc42ff3277bc43dbb3102ea',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5ftime',['libinput_event_pointer_get_time',['../group__event__pointer.html#gac42d0e19d076f4e1fca15f1ec42510d5',1,'libinput.h']]],
+  ['libinput_5fevent_5fpointer_5fget_5ftime_5fusec',['libinput_event_pointer_get_time_usec',['../group__event__pointer.html#gaf0997d730299f40dbe3efcfa06c429be',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fhas_5faxis',['libinput_event_pointer_has_axis',['../group__event__pointer.html#ga1fdd37a49eae958aaf78a0596cd5cef5',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5fbase_5fevent',['libinput_event_touch_get_base_event',['../group__event__touch.html#gacb2cd144a719c9873cd4621db278e92c',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5fseat_5fslot',['libinput_event_touch_get_seat_slot',['../group__event__touch.html#gabac75c78a0a360995e1a521edb38c4da',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5fslot',['libinput_event_touch_get_slot',['../group__event__touch.html#gaf5294eab86b9125b7fb607049ff9ef09',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5ftime',['libinput_event_touch_get_time',['../group__event__touch.html#ga5d13da5020c6aff78604943690f8cbe9',1,'libinput.h']]],
+  ['libinput_5fevent_5ftouch_5fget_5ftime_5fusec',['libinput_event_touch_get_time_usec',['../group__event__touch.html#ga83cc90b988d15037c438f5c457feab5e',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5fx',['libinput_event_touch_get_x',['../group__event__touch.html#gac42d0e070811c360cf8ec784349954ba',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5fx_5ftransformed',['libinput_event_touch_get_x_transformed',['../group__event__touch.html#gadd04ccc3a7b4564e5aca3c8592f090fe',1,'libinput.h']]],
   ['libinput_5fevent_5ftouch_5fget_5fy',['libinput_event_touch_get_y',['../group__event__touch.html#ga30711113bcef4bc02eda7878a054351c',1,'libinput.h']]],
