@@ -129,6 +129,19 @@ enum tp_edge_scroll_touch_state {
 	EDGE_SCROLL_TOUCH_STATE_AREA,
 };
 
+enum tp_gesture_2fg_state {
+	GESTURE_2FG_STATE_NONE,
+	GESTURE_2FG_STATE_UNKNOWN,
+	GESTURE_2FG_STATE_SCROLL,
+	GESTURE_2FG_STATE_PINCH,
+};
+
+enum tp_thumb_state {
+	THUMB_STATE_NO,
+	THUMB_STATE_YES,
+	THUMB_STATE_MAYBE,
+};
+
 struct tp_touch {
 	struct tp_dispatch *tp;
 	enum touch_state state;
@@ -137,6 +150,16 @@ struct tp_touch {
 	struct device_coords point;
 	uint64_t millis;
 	int distance;				/* distance == 0 means touch */
+	int pressure;
+
+	struct {
+		/* A quirk mostly used on Synaptics touchpads. In a
+		   transition to/from fake touches > num_slots, the current
+		   event data is likely garbage and the subsequent event
+		   is likely too. This marker tells us to reset the motion
+		   history again -> this effectively swallows any motion */
+		bool reset_motion_history;
+	} quirks;
 
 	struct {
 		struct device_coords samples[TOUCHPAD_HISTORY_LENGTH];
@@ -166,6 +189,7 @@ struct tp_touch {
 	struct {
 		enum tp_tap_touch_state state;
 		struct device_coords initial;
+		bool is_thumb;
 	} tap;
 
 	struct {
@@ -179,8 +203,18 @@ struct tp_touch {
 	struct {
 		enum touch_palm_state state;
 		struct device_coords first; /* first coordinates if is_palm == true */
-		uint32_t time; /* first timestamp if is_palm == true */
+		uint64_t time; /* first timestamp if is_palm == true */
 	} palm;
+
+	struct {
+		struct device_coords initial;
+	} gesture;
+
+	struct {
+		enum tp_thumb_state state;
+		uint64_t first_touch_time;
+		struct device_coords initial;
+	} thumb;
 };
 
 struct tp_dispatch {
@@ -212,10 +246,18 @@ struct tp_dispatch {
 	} accel;
 
 	struct {
+		bool enabled;
 		bool started;
 		unsigned int finger_count;
 		unsigned int finger_count_pending;
 		struct libinput_timer finger_count_switch_timer;
+		enum tp_gesture_2fg_state twofinger_state;
+		struct tp_touch *touches[2];
+		uint64_t initial_time;
+		double initial_distance;
+		double prev_scale;
+		double angle;
+		struct device_float_coords center;
 	} gesture;
 
 	struct {
@@ -292,6 +334,9 @@ struct tp_dispatch {
 	} sendevents;
 
 	struct {
+		struct libinput_device_config_dwt config;
+		bool dwt_enabled;
+
 		bool keyboard_active;
 		struct libinput_event_listener keyboard_listener;
 		struct libinput_timer keyboard_timer;
@@ -299,6 +344,13 @@ struct tp_dispatch {
 
 		uint64_t keyboard_last_press_time;
 	} dwt;
+
+	struct {
+		bool detect_thumbs;
+		int threshold;
+		int upper_thumb_line;
+		int lower_thumb_line;
+	} thumb;
 };
 
 #define tp_for_each_touch(_tp, _t) \
@@ -343,6 +395,10 @@ struct normalized_coords
 tp_filter_motion(struct tp_dispatch *tp,
 		 const struct normalized_coords *unaccelerated,
 		 uint64_t time);
+struct normalized_coords
+tp_filter_motion_unaccelerated(struct tp_dispatch *tp,
+			       const struct normalized_coords *unaccelerated,
+			       uint64_t time);
 
 int
 tp_touch_active(struct tp_dispatch *tp, struct tp_touch *t);
@@ -431,6 +487,9 @@ tp_remove_gesture(struct tp_dispatch *tp);
 void
 tp_gesture_stop(struct tp_dispatch *tp, uint64_t time);
 
+void
+tp_gesture_cancel(struct tp_dispatch *tp, uint64_t time);
+
 void
 tp_gesture_handle_state(struct tp_dispatch *tp, uint64_t time);
 
