@@ -70,7 +70,8 @@ START_TEST(device_sendevents_config_touchpad)
 	expected = LIBINPUT_CONFIG_SEND_EVENTS_DISABLED;
 
 	/* The wacom devices in the test suite are external */
-	if (libevdev_get_id_vendor(dev->evdev) != VENDOR_ID_WACOM)
+	if (libevdev_get_id_vendor(dev->evdev) != VENDOR_ID_WACOM &&
+	    libevdev_get_id_bustype(dev->evdev) != BUS_BLUETOOTH)
 		expected |=
 			LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE;
 
@@ -89,7 +90,8 @@ START_TEST(device_sendevents_config_touchpad_superset)
 	uint32_t modes;
 
 	/* The wacom devices in the test suite are external */
-	if (libevdev_get_id_vendor(dev->evdev) == 0x56a) /* wacom */
+	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_WACOM ||
+	    libevdev_get_id_bustype(dev->evdev) == BUS_BLUETOOTH)
 		return;
 
 	device = dev->libinput_device;
@@ -1006,6 +1008,178 @@ START_TEST(device_udev_tag_apple)
 	udev_device_unref(d);
 }
 END_TEST
+
+START_TEST(device_udev_tag_synaptics_serial)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	struct udev_device *d;
+	const char *prop;
+
+	d = libinput_device_get_udev_device(device);
+	prop = udev_device_get_property_value(d,
+					      "LIBINPUT_MODEL_SYNAPTICS_SERIAL_TOUCHPAD");
+
+	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_SYNAPTICS_SERIAL &&
+	    libevdev_get_id_product(dev->evdev) == PRODUCT_ID_SYNAPTICS_SERIAL)
+		ck_assert_notnull(prop);
+	else
+		ck_assert(prop == NULL);
+
+	udev_device_unref(d);
+}
+END_TEST
+
+START_TEST(device_nonpointer_rel)
+{
+	struct libevdev_uinput *uinput;
+	struct libinput *li;
+	struct libinput_device *device;
+	int i;
+
+	uinput = litest_create_uinput_device("test device",
+					     NULL,
+					     EV_KEY, KEY_A,
+					     EV_KEY, KEY_B,
+					     EV_REL, REL_X,
+					     EV_REL, REL_Y,
+					     -1);
+	li = litest_create_context();
+	device = libinput_path_add_device(li,
+					  libevdev_uinput_get_devnode(uinput));
+	ck_assert(device != NULL);
+
+	litest_disable_log_handler(li);
+	for (i = 0; i < 100; i++) {
+		libevdev_uinput_write_event(uinput, EV_REL, REL_X, 1);
+		libevdev_uinput_write_event(uinput, EV_REL, REL_Y, -1);
+		libevdev_uinput_write_event(uinput, EV_SYN, SYN_REPORT, 0);
+		libinput_dispatch(li);
+	}
+	litest_restore_log_handler(li);
+
+	libinput_unref(li);
+	libevdev_uinput_destroy(uinput);
+}
+END_TEST
+
+START_TEST(device_touchpad_rel)
+{
+	struct libevdev_uinput *uinput;
+	struct libinput *li;
+	struct libinput_device *device;
+	const struct input_absinfo abs[] = {
+		{ ABS_X, 0, 10, 0, 0, 10 },
+		{ ABS_Y, 0, 10, 0, 0, 10 },
+		{ ABS_MT_SLOT, 0, 2, 0, 0, 0 },
+		{ ABS_MT_TRACKING_ID, 0, 255, 0, 0, 0 },
+		{ ABS_MT_POSITION_X, 0, 10, 0, 0, 10 },
+		{ ABS_MT_POSITION_Y, 0, 10, 0, 0, 10 },
+		{ -1, -1, -1, -1, -1, -1 }
+	};
+	int i;
+
+	uinput = litest_create_uinput_abs_device("test device",
+						 NULL, abs,
+						 EV_KEY, BTN_TOOL_FINGER,
+						 EV_KEY, BTN_TOUCH,
+						 EV_REL, REL_X,
+						 EV_REL, REL_Y,
+						 -1);
+	li = litest_create_context();
+	device = libinput_path_add_device(li,
+					  libevdev_uinput_get_devnode(uinput));
+	ck_assert(device != NULL);
+
+	for (i = 0; i < 100; i++) {
+		libevdev_uinput_write_event(uinput, EV_REL, REL_X, 1);
+		libevdev_uinput_write_event(uinput, EV_REL, REL_Y, -1);
+		libevdev_uinput_write_event(uinput, EV_SYN, SYN_REPORT, 0);
+		libinput_dispatch(li);
+	}
+
+	libinput_unref(li);
+	libevdev_uinput_destroy(uinput);
+}
+END_TEST
+
+START_TEST(device_touch_rel)
+{
+	struct libevdev_uinput *uinput;
+	struct libinput *li;
+	struct libinput_device *device;
+	const struct input_absinfo abs[] = {
+		{ ABS_X, 0, 10, 0, 0, 10 },
+		{ ABS_Y, 0, 10, 0, 0, 10 },
+		{ ABS_MT_SLOT, 0, 2, 0, 0, 0 },
+		{ ABS_MT_TRACKING_ID, 0, 255, 0, 0, 0 },
+		{ ABS_MT_POSITION_X, 0, 10, 0, 0, 10 },
+		{ ABS_MT_POSITION_Y, 0, 10, 0, 0, 10 },
+		{ -1, -1, -1, -1, -1, -1 }
+	};
+	int i;
+
+	uinput = litest_create_uinput_abs_device("test device",
+						 NULL, abs,
+						 EV_KEY, BTN_TOUCH,
+						 EV_REL, REL_X,
+						 EV_REL, REL_Y,
+						 -1);
+	li = litest_create_context();
+	device = libinput_path_add_device(li,
+					  libevdev_uinput_get_devnode(uinput));
+	ck_assert(device != NULL);
+
+	litest_disable_log_handler(li);
+	for (i = 0; i < 100; i++) {
+		libevdev_uinput_write_event(uinput, EV_REL, REL_X, 1);
+		libevdev_uinput_write_event(uinput, EV_REL, REL_Y, -1);
+		libevdev_uinput_write_event(uinput, EV_SYN, SYN_REPORT, 0);
+		libinput_dispatch(li);
+	}
+	litest_restore_log_handler(li);
+
+	libinput_unref(li);
+	libevdev_uinput_destroy(uinput);
+}
+END_TEST
+
+START_TEST(device_abs_rel)
+{
+	struct libevdev_uinput *uinput;
+	struct libinput *li;
+	struct libinput_device *device;
+	const struct input_absinfo abs[] = {
+		{ ABS_X, 0, 10, 0, 0, 10 },
+		{ ABS_Y, 0, 10, 0, 0, 10 },
+		{ -1, -1, -1, -1, -1, -1 }
+	};
+	int i;
+
+	uinput = litest_create_uinput_abs_device("test device",
+						 NULL, abs,
+						 EV_KEY, BTN_TOUCH,
+						 EV_KEY, BTN_LEFT,
+						 EV_REL, REL_X,
+						 EV_REL, REL_Y,
+						 -1);
+	li = litest_create_context();
+	device = libinput_path_add_device(li,
+					  libevdev_uinput_get_devnode(uinput));
+	ck_assert(device != NULL);
+
+	for (i = 0; i < 100; i++) {
+		libevdev_uinput_write_event(uinput, EV_REL, REL_X, 1);
+		libevdev_uinput_write_event(uinput, EV_REL, REL_Y, -1);
+		libevdev_uinput_write_event(uinput, EV_SYN, SYN_REPORT, 0);
+		libinput_dispatch(li);
+	}
+
+	libinput_unref(li);
+	libevdev_uinput_destroy(uinput);
+}
+END_TEST
+
 void
 litest_setup_tests(void)
 {
@@ -1052,4 +1226,10 @@ litest_setup_tests(void)
 	litest_add("device:udev tags", device_udev_tag_alps, LITEST_TOUCHPAD, LITEST_ANY);
 	litest_add("device:udev tags", device_udev_tag_wacom, LITEST_TOUCHPAD, LITEST_ANY);
 	litest_add("device:udev tags", device_udev_tag_apple, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add("device:udev tags", device_udev_tag_synaptics_serial, LITEST_TOUCHPAD, LITEST_ANY);
+
+	litest_add_no_device("device:invalid rel events", device_nonpointer_rel);
+	litest_add_no_device("device:invalid rel events", device_touchpad_rel);
+	litest_add_no_device("device:invalid rel events", device_touch_rel);
+	litest_add_no_device("device:invalid rel events", device_abs_rel);
 }
