@@ -79,6 +79,12 @@ struct litest_device_interface {
 	void (*touch_move)(struct litest_device *d, unsigned int slot, double x, double y);
 	void (*touch_up)(struct litest_device *d, unsigned int slot);
 
+	/**
+	 * Default value for the given EV_ABS axis.
+	 * @return 0 on success, nonzero otherwise
+	 */
+	int (*get_axis_default)(struct litest_device *d, unsigned int code, int32_t *value);
+
 	/**
 	 * Set of of events to execute on touch down, terminated by a .type
 	 * and .code value of -1. If the event value is LITEST_AUTO_ASSIGN,
@@ -91,8 +97,8 @@ struct litest_device_interface {
 	struct input_event *touch_move_events;
 	struct input_event *touch_up_events;
 
-	int min[2];
-	int max[2];
+	int min[2]; /* x/y axis minimum */
+	int max[2]; /* x/y axis maximum */
 };
 
 void litest_set_current_device(struct litest_device *device);
