@@ -44,6 +44,8 @@
 #include <sys/sendfile.h>
 #include <sys/timerfd.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <libudev.h>
 
 #include "litest.h"
@@ -53,8 +55,12 @@
 #define UDEV_RULES_D "/run/udev/rules.d"
 #define UDEV_RULE_PREFIX "99-litest-"
 #define UDEV_HWDB_D "/etc/udev/hwdb.d"
-#define UDEV_COMMON_RULE_FILE UDEV_RULES_D "/91-litest-model-quirks-REMOVEME.rules"
-#define UDEV_COMMON_HWDB_FILE UDEV_HWDB_D "/91-litest-model-quirks-REMOVEME.hwdb"
+#define UDEV_MODEL_QUIRKS_RULE_FILE UDEV_RULES_D \
+	"/91-litest-model-quirks-REMOVEME.rules"
+#define UDEV_MODEL_QUIRKS_HWDB_FILE UDEV_HWDB_D \
+	"/91-litest-model-quirks-REMOVEME.hwdb"
+#define UDEV_TEST_DEVICE_RULE_FILE UDEV_RULES_D \
+	"/91-litest-test-device-REMOVEME.rules"
 
 static int in_debugger = -1;
 static int verbose = 0;
@@ -63,6 +69,7 @@ const char *filter_device = NULL;
 const char *filter_group = NULL;
 
 static inline void litest_remove_model_quirks(void);
+static void litest_init_udev_rules(void);
 
 /* defined for the litest selftest */
 #ifndef LITEST_DISABLE_BACKTRACE_LOGGING
@@ -214,13 +221,13 @@ litest_backtrace(void)
 		}
 
 		if (have_lineno) {
-			litest_log("%u: %s() (%s:%d)\n",
+			litest_log("%d: %s() (%s:%d)\n",
 				   i,
 				   procname,
 				   file,
 				   line);
 		} else  {
-			litest_log("%u: %s (%s%s+%#x) [%p]\n",
+			litest_log("%d: %s (%s%s+%#x) [%p]\n",
 				   i,
 				   filename,
 				   procname,
@@ -354,6 +361,10 @@ extern struct litest_test_device litest_logitech_trackball_device;
 extern struct litest_test_device litest_atmel_hover_device;
 extern struct litest_test_device litest_alps_dualpoint_device;
 extern struct litest_test_device litest_mouse_low_dpi_device;
+extern struct litest_test_device litest_generic_multitouch_screen_device;
+extern struct litest_test_device litest_nexus4_device;
+extern struct litest_test_device litest_magicpad_device;
+extern struct litest_test_device litest_elantech_touchpad_device;
 
 struct litest_test_device* devices[] = {
 	&litest_synaptics_clickpad_device,
@@ -381,6 +392,10 @@ struct litest_test_device* devices[] = {
 	&litest_atmel_hover_device,
 	&litest_alps_dualpoint_device,
 	&litest_mouse_low_dpi_device,
+	&litest_generic_multitouch_screen_device,
+	&litest_nexus4_device,
+	&litest_magicpad_device,
+	&litest_elantech_touchpad_device,
 	NULL,
 };
 
@@ -434,7 +449,7 @@ litest_drop_udev_rules(void)
 		    &entries,
 		    litest_udev_rule_filter,
 		    alphasort);
-	if (n < 0)
+	if (n <= 0)
 		return;
 
 	while (n--) {
@@ -454,7 +469,6 @@ litest_drop_udev_rules(void)
 	}
 	free(entries);
 
-	litest_remove_model_quirks();
 	litest_reload_udev_rules();
 }
 
@@ -765,6 +779,12 @@ litest_log_handler(struct libinput *libinput,
 
 	fprintf(stderr, "litest %s: ", priority);
 	vfprintf(stderr, format, args);
+
+#if 0
+	if (strstr(format, "client bug: ") ||
+	    strstr(format, "libinput bug: "))
+		litest_abort_msg("libinput bug triggered, aborting.\n");
+#endif
 }
 
 static int
@@ -814,6 +834,8 @@ litest_run(int argc, char **argv)
 	if (getenv("LITEST_VERBOSE"))
 		verbose = 1;
 
+	litest_init_udev_rules();
+
 	srunner_run_all(sr, CK_ENV);
 	failed = srunner_ntests_failed(sr);
 	srunner_free(sr);
@@ -832,6 +854,9 @@ litest_run(int argc, char **argv)
 		free(s);
 	}
 
+	litest_remove_model_quirks();
+	litest_reload_udev_rules();
+
 	return failed;
 }
 
@@ -934,27 +959,29 @@ litest_install_model_quirks(void)
 			 "# running, remove this file and update your hwdb: \n"
 			 "#       sudo udevadm hwdb --update\n"
 			 "#################################################################\n\n";
-	litest_copy_file(UDEV_COMMON_RULE_FILE,
-			 LIBINPUT_UDEV_RULES_FILE,
+	litest_copy_file(UDEV_MODEL_QUIRKS_RULE_FILE,
+			 LIBINPUT_MODEL_QUIRKS_UDEV_RULES_FILE,
 			 warning);
-	litest_copy_file(UDEV_COMMON_HWDB_FILE,
-			 LIBINPUT_UDEV_HWDB_FILE,
+	litest_copy_file(UDEV_MODEL_QUIRKS_HWDB_FILE,
+			 LIBINPUT_MODEL_QUIRKS_UDEV_HWDB_FILE,
+			 warning);
+	litest_copy_file(UDEV_TEST_DEVICE_RULE_FILE,
+			 LIBINPUT_TEST_DEVICE_RULES_FILE,
 			 warning);
 }
 
 static inline void
 litest_remove_model_quirks(void)
 {
-	unlink(UDEV_COMMON_RULE_FILE);
-	unlink(UDEV_COMMON_HWDB_FILE);
+	unlink(UDEV_MODEL_QUIRKS_RULE_FILE);
+	unlink(UDEV_MODEL_QUIRKS_HWDB_FILE);
+	unlink(UDEV_TEST_DEVICE_RULE_FILE);
 }
 
-static char *
-litest_init_udev_rules(struct litest_test_device *dev)
+static void
+litest_init_udev_rules(void)
 {
 	int rc;
-	FILE *f;
-	char *path = NULL;
 
 	rc = mkdir(UDEV_RULES_D, 0755);
 	if (rc == -1 && errno != EEXIST)
@@ -967,10 +994,18 @@ litest_init_udev_rules(struct litest_test_device *dev)
 			     strerror(errno));
 
 	litest_install_model_quirks();
+	litest_reload_udev_rules();
+}
+
+static char *
+litest_init_device_udev_rules(struct litest_test_device *dev)
+{
+	int rc;
+	FILE *f;
+	char *path = NULL;
 
-	/* device-specific udev rules */
 	if (!dev->udev_rule)
-		goto out;
+		return NULL;
 
 	rc = xasprintf(&path,
 		      "%s/%s%s.rules",
@@ -987,7 +1022,6 @@ litest_init_udev_rules(struct litest_test_device *dev)
 	litest_assert_int_ge(fputs(dev->udev_rule, f), 0);
 	fclose(f);
 
-out:
 	litest_reload_udev_rules();
 
 	return path;
@@ -1021,18 +1055,17 @@ litest_create(enum litest_device_type which,
 	d = zalloc(sizeof(*d));
 	litest_assert(d != NULL);
 
-	udev_file = litest_init_udev_rules(*dev);
-
+	udev_file = litest_init_device_udev_rules(*dev);
 	/* device has custom create method */
 	if ((*dev)->create) {
 		(*dev)->create(d);
 		if (abs_override || events_override) {
-			litest_remove_model_quirks();
 			if (udev_file)
 				unlink(udev_file);
 			litest_abort_msg("Custom create cannot be overridden");
 		}
 
+		d->udev_rule_file = udev_file;
 		return d;
 	}
 
@@ -1178,10 +1211,10 @@ litest_delete_device(struct litest_device *d)
 		return;
 
 	if (d->udev_rule_file) {
-		litest_remove_model_quirks();
 		unlink(d->udev_rule_file);
 		free(d->udev_rule_file);
 		d->udev_rule_file = NULL;
+		litest_reload_udev_rules();
 	}
 
 	libinput_device_unref(d->libinput_device);
@@ -1208,10 +1241,32 @@ litest_event(struct litest_device *d, unsigned int type,
 	litest_assert_int_eq(ret, 0);
 }
 
+static bool
+axis_replacement_value(struct axis_replacement *axes,
+		       int32_t evcode,
+		       int32_t *value)
+{
+	struct axis_replacement *axis = axes;
+
+	if (!axes)
+		return false;
+
+	while (axis->evcode != -1) {
+		if (axis->evcode == evcode) {
+			*value = axis->value;
+			return true;
+		}
+		axis++;
+	}
+
+	return false;
+}
+
 int
 litest_auto_assign_value(struct litest_device *d,
 			 const struct input_event *ev,
 			 int slot, double x, double y,
+			 struct axis_replacement *axes,
 			 bool touching)
 {
 	static int tracking_id;
@@ -1238,6 +1293,12 @@ litest_auto_assign_value(struct litest_device *d,
 	case ABS_MT_DISTANCE:
 		value = touching ? 0 : 1;
 		break;
+	default:
+		value = -1;
+		if (!axis_replacement_value(axes, ev->code, &value) &&
+		    d->interface->get_axis_default)
+			d->interface->get_axis_default(d, ev->code, &value);
+		break;
 	}
 
 	return value;
@@ -1255,8 +1316,12 @@ send_btntool(struct litest_device *d)
 }
 
 static void
-litest_slot_start(struct litest_device *d, unsigned int slot,
-		  double x, double y, bool touching)
+litest_slot_start(struct litest_device *d,
+		  unsigned int slot,
+		  double x,
+		  double y,
+		  struct axis_replacement *axes,
+		  bool touching)
 {
 	struct input_event *ev;
 
@@ -1277,6 +1342,7 @@ litest_slot_start(struct litest_device *d, unsigned int slot,
 						     slot,
 						     x,
 						     y,
+						     axes,
 						     touching);
 
 		litest_event(d, ev->type, ev->code, value);
@@ -1285,10 +1351,22 @@ litest_slot_start(struct litest_device *d, unsigned int slot,
 }
 
 void
-litest_touch_down(struct litest_device *d, unsigned int slot,
-		  double x, double y)
+litest_touch_down(struct litest_device *d,
+		  unsigned int slot,
+		  double x,
+		  double y)
 {
-	litest_slot_start(d, slot, x, y, 1);
+	litest_slot_start(d, slot, x, y, NULL, true);
+}
+
+void
+litest_touch_down_extended(struct litest_device *d,
+			   unsigned int slot,
+			   double x,
+			   double y,
+			   struct axis_replacement *axes)
+{
+	litest_slot_start(d, slot, x, y, axes, true);
 }
 
 void
@@ -1321,6 +1399,7 @@ litest_touch_up(struct litest_device *d, unsigned int slot)
 						     slot,
 						     0,
 						     0,
+						     NULL,
 						     false);
 		litest_event(d, ev->type, ev->code, value);
 		ev++;
@@ -1328,8 +1407,12 @@ litest_touch_up(struct litest_device *d, unsigned int slot)
 }
 
 static void
-litest_slot_move(struct litest_device *d, unsigned int slot,
-		 double x, double y, bool touching)
+litest_slot_move(struct litest_device *d,
+		 unsigned int slot,
+		 double x,
+		 double y,
+		 struct axis_replacement *axes,
+		 bool touching)
 {
 	struct input_event *ev;
 
@@ -1345,6 +1428,7 @@ litest_slot_move(struct litest_device *d, unsigned int slot,
 						     slot,
 						     x,
 						     y,
+						     axes,
 						     touching);
 		litest_event(d, ev->type, ev->code, value);
 		ev++;
@@ -1352,10 +1436,22 @@ litest_slot_move(struct litest_device *d, unsigned int slot,
 }
 
 void
-litest_touch_move(struct litest_device *d, unsigned int slot,
-		  double x, double y)
+litest_touch_move(struct litest_device *d,
+		  unsigned int slot,
+		  double x,
+		  double y)
 {
-	litest_slot_move(d, slot, x, y, true);
+	litest_slot_move(d, slot, x, y, NULL, true);
+}
+
+void
+litest_touch_move_extended(struct litest_device *d,
+			   unsigned int slot,
+			   double x,
+			   double y,
+			   struct axis_replacement *axes)
+{
+	litest_slot_move(d, slot, x, y, axes, true);
 }
 
 void
@@ -1401,10 +1497,38 @@ litest_touch_move_two_touches(struct litest_device *d,
 }
 
 void
-litest_hover_start(struct litest_device *d, unsigned int slot,
-		   double x, double y)
+litest_touch_move_three_touches(struct litest_device *d,
+				double x0, double y0,
+				double x1, double y1,
+				double x2, double y2,
+				double dx, double dy,
+				int steps, int sleep_ms)
 {
-	litest_slot_start(d, slot, x, y, 0);
+	for (int i = 0; i < steps - 1; i++) {
+		litest_touch_move(d, 0, x0 + dx / steps * i,
+					y0 + dy / steps * i);
+		litest_touch_move(d, 1, x1 + dx / steps * i,
+					y1 + dy / steps * i);
+		litest_touch_move(d, 2, x2 + dx / steps * i,
+					y2 + dy / steps * i);
+		if (sleep_ms) {
+			libinput_dispatch(d->libinput);
+			msleep(sleep_ms);
+			libinput_dispatch(d->libinput);
+		}
+	}
+	litest_touch_move(d, 0, x0 + dx, y0 + dy);
+	litest_touch_move(d, 1, x1 + dx, y1 + dy);
+	litest_touch_move(d, 2, x2 + dx, y2 + dy);
+}
+
+void
+litest_hover_start(struct litest_device *d,
+		   unsigned int slot,
+		   double x,
+		   double y)
+{
+	litest_slot_start(d, slot, x, y, NULL, 0);
 }
 
 void
@@ -1433,7 +1557,7 @@ litest_hover_end(struct litest_device *d, unsigned int slot)
 		ev = up;
 
 	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
-		int value = litest_auto_assign_value(d, ev, slot, 0, 0, 0);
+		int value = litest_auto_assign_value(d, ev, slot, 0, 0, NULL, false);
 		litest_event(d, ev->type, ev->code, value);
 		ev++;
 	}
@@ -1443,7 +1567,7 @@ void
 litest_hover_move(struct litest_device *d, unsigned int slot,
 		  double x, double y)
 {
-	litest_slot_move(d, slot, x, y, false);
+	litest_slot_move(d, slot, x, y, NULL, false);
 }
 
 void
@@ -1655,6 +1779,24 @@ litest_event_type_str(struct libinput_event *event)
 	case LIBINPUT_EVENT_TOUCH_FRAME:
 		str = "TOUCH FRAME";
 		break;
+	case LIBINPUT_EVENT_GESTURE_SWIPE_BEGIN:
+		str = "GESTURE SWIPE START";
+		break;
+	case LIBINPUT_EVENT_GESTURE_SWIPE_UPDATE:
+		str = "GESTURE SWIPE UPDATE";
+		break;
+	case LIBINPUT_EVENT_GESTURE_SWIPE_END:
+		str = "GESTURE SWIPE END";
+		break;
+	case LIBINPUT_EVENT_GESTURE_PINCH_BEGIN:
+		str = "GESTURE PINCH START";
+		break;
+	case LIBINPUT_EVENT_GESTURE_PINCH_UPDATE:
+		str = "GESTURE PINCH UPDATE";
+		break;
+	case LIBINPUT_EVENT_GESTURE_PINCH_END:
+		str = "GESTURE PINCH END";
+		break;
 	}
 	return str;
 }
@@ -1696,12 +1838,17 @@ litest_print_event(struct libinput_event *event)
 		break;
 	case LIBINPUT_EVENT_POINTER_AXIS:
 		p = libinput_event_get_pointer_event(event);
-		fprintf(stderr,
-			"vert %.f horiz %.2f",
-			libinput_event_pointer_get_axis_value(p,
-				LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL),
-			libinput_event_pointer_get_axis_value(p,
-				LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+		x = 0.0;
+		y = 0.0;
+		if (libinput_event_pointer_has_axis(p,
+				LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL))
+			y = libinput_event_pointer_get_axis_value(p,
+				LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL);
+		if (libinput_event_pointer_has_axis(p,
+				LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL))
+			x = libinput_event_pointer_get_axis_value(p,
+				LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL);
+		fprintf(stderr, "vert %.f horiz %.2f", y, x);
 		break;
 	default:
 		break;
@@ -1835,6 +1982,7 @@ litest_create_uinput_device_from_description(const char *name,
 	struct udev_device *udev_device;
 	const char *udev_action;
 	const char *udev_syspath = NULL;
+	int rc;
 
 	udev = udev_new();
 	litest_assert_notnull(udev);
@@ -1843,7 +1991,8 @@ litest_create_uinput_device_from_description(const char *name,
 	udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "input",
 							NULL);
 	/* remove O_NONBLOCK */
-	fcntl(udev_monitor_get_fd(udev_monitor), F_SETFL, 0);
+	rc = fcntl(udev_monitor_get_fd(udev_monitor), F_SETFL, 0);
+	litest_assert_int_ne(rc, -1);
 	litest_assert_int_eq(udev_monitor_enable_receiving(udev_monitor),
 			     0);
 
@@ -2049,6 +2198,25 @@ litest_is_keyboard_event(struct libinput_event *event,
 	return kevent;
 }
 
+struct libinput_event_gesture *
+litest_is_gesture_event(struct libinput_event *event,
+			enum libinput_event_type type,
+			int nfingers)
+{
+	struct libinput_event_gesture *gevent;
+
+	litest_assert(event != NULL);
+	litest_assert_int_eq(libinput_event_get_type(event), type);
+
+	gevent = libinput_event_get_gesture_event(event);
+	litest_assert(gevent != NULL);
+
+	if (nfingers != -1)
+		litest_assert_int_eq(libinput_event_gesture_get_finger_count(gevent),
+				     nfingers);
+	return gevent;
+}
+
 void
 litest_assert_scroll(struct libinput *li,
 		     enum libinput_pointer_axis axis,
@@ -2076,9 +2244,9 @@ litest_assert_scroll(struct libinput *li,
 			}
 		} else {
 			/* Last scroll event, must be 0 */
-			litest_assert_int_eq(
+			ck_assert_double_eq(
 				libinput_event_pointer_get_axis_value(ptrev, axis),
-				0);
+				0.0);
 		}
 		libinput_event_destroy(event);
 		event = next_event;
@@ -2161,6 +2329,12 @@ litest_timeout_dwt_long(void)
 	msleep(520);
 }
 
+void
+litest_timeout_gesture(void)
+{
+	msleep(120);
+}
+
 void
 litest_push_event_frame(struct litest_device *dev)
 {
@@ -2185,11 +2359,11 @@ send_abs_xy(struct litest_device *d, double x, double y)
 	e.type = EV_ABS;
 	e.code = ABS_X;
 	e.value = LITEST_AUTO_ASSIGN;
-	val = litest_auto_assign_value(d, &e, 0, x, y, true);
+	val = litest_auto_assign_value(d, &e, 0, x, y, NULL, true);
 	litest_event(d, EV_ABS, ABS_X, val);
 
 	e.code = ABS_Y;
-	val = litest_auto_assign_value(d, &e, 0, x, y, true);
+	val = litest_auto_assign_value(d, &e, 0, x, y, NULL, true);
 	litest_event(d, EV_ABS, ABS_Y, val);
 }
 
@@ -2202,12 +2376,12 @@ send_abs_mt_xy(struct litest_device *d, double x, double y)
 	e.type = EV_ABS;
 	e.code = ABS_MT_POSITION_X;
 	e.value = LITEST_AUTO_ASSIGN;
-	val = litest_auto_assign_value(d, &e, 0, x, y, true);
+	val = litest_auto_assign_value(d, &e, 0, x, y, NULL, true);
 	litest_event(d, EV_ABS, ABS_MT_POSITION_X, val);
 
 	e.code = ABS_MT_POSITION_Y;
 	e.value = LITEST_AUTO_ASSIGN;
-	val = litest_auto_assign_value(d, &e, 0, x, y, true);
+	val = litest_auto_assign_value(d, &e, 0, x, y, NULL, true);
 	litest_event(d, EV_ABS, ABS_MT_POSITION_Y, val);
 }
 
@@ -2396,6 +2570,7 @@ main(int argc, char **argv)
 	list_init(&all_tests);
 
 	setenv("CK_DEFAULT_TIMEOUT", "10", 0);
+	setenv("LIBINPUT_RUNNING_TEST_SUITE", "1", 1);
 
 	mode = litest_parse_argv(argc, argv);
 	if (mode == LITEST_MODE_ERROR)
