@@ -197,7 +197,7 @@ START_TEST(pointer_motion_relative_min_decel)
 	ck_assert((evx == 0.0) == (dx == 0));
 	ck_assert((evy == 0.0) == (dy == 0));
 
-	len = vector_length(evx, evy);
+	len = hypot(evx, evy);
 	ck_assert(fabs(len) >= 0.3);
 
 	libinput_event_destroy(event);
@@ -251,7 +251,6 @@ START_TEST(pointer_absolute_initial_state)
 	struct libinput_event_pointer *p1, *p2;
 	int axis = _i; /* looped test */
 
-	dev = litest_current_device();
 	libinput1 = dev->libinput;
 	litest_touch_down(dev, 0, 40, 60);
 	litest_touch_up(dev, 0);
@@ -898,15 +897,18 @@ START_TEST(pointer_accel_defaults)
 	double speed;
 
 	ck_assert(libinput_device_config_accel_is_available(device));
-	ck_assert(libinput_device_config_accel_get_default_speed(device) == 0.0);
-	ck_assert(libinput_device_config_accel_get_speed(device) == 0.0);
+	ck_assert_double_eq(libinput_device_config_accel_get_default_speed(device),
+			    0.0);
+	ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+			    0.0);
 
 	for (speed = -2.0; speed < -1.0; speed += 0.2) {
 		status = libinput_device_config_accel_set_speed(device,
 								speed);
 		ck_assert_int_eq(status,
 				 LIBINPUT_CONFIG_STATUS_INVALID);
-		ck_assert(libinput_device_config_accel_get_speed(device) == 0.0);
+		ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+				    0.0);
 	}
 
 	for (speed = -1.0; speed <= 1.0; speed += 0.2) {
@@ -914,7 +916,8 @@ START_TEST(pointer_accel_defaults)
 								speed);
 		ck_assert_int_eq(status,
 				 LIBINPUT_CONFIG_STATUS_SUCCESS);
-		ck_assert(libinput_device_config_accel_get_speed(device) == speed);
+		ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+				    speed);
 	}
 
 	for (speed = 1.2; speed <= -2.0; speed += 0.2) {
@@ -922,7 +925,8 @@ START_TEST(pointer_accel_defaults)
 								speed);
 		ck_assert_int_eq(status,
 				 LIBINPUT_CONFIG_STATUS_INVALID);
-		ck_assert(libinput_device_config_accel_get_speed(device) == 1.0);
+		ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+				    1.0);
 	}
 
 }
@@ -953,8 +957,10 @@ START_TEST(pointer_accel_defaults_absolute)
 	double speed;
 
 	ck_assert(!libinput_device_config_accel_is_available(device));
-	ck_assert(libinput_device_config_accel_get_default_speed(device) == 0.0);
-	ck_assert(libinput_device_config_accel_get_speed(device) == 0.0);
+	ck_assert_double_eq(libinput_device_config_accel_get_default_speed(device),
+			    0.0);
+	ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+			    0.0);
 
 	for (speed = -2.0; speed <= 2.0; speed += 0.2) {
 		status = libinput_device_config_accel_set_speed(device,
@@ -965,7 +971,8 @@ START_TEST(pointer_accel_defaults_absolute)
 		else
 			ck_assert_int_eq(status,
 					 LIBINPUT_CONFIG_STATUS_INVALID);
-		ck_assert(libinput_device_config_accel_get_speed(device) == 0.0);
+		ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+				    0.0);
 	}
 }
 END_TEST
@@ -976,8 +983,10 @@ START_TEST(pointer_accel_defaults_absolute_relative)
 	struct libinput_device *device = dev->libinput_device;
 
 	ck_assert(libinput_device_config_accel_is_available(device));
-	ck_assert(libinput_device_config_accel_get_default_speed(device) == 0.0);
-	ck_assert(libinput_device_config_accel_get_speed(device) == 0.0);
+	ck_assert_double_eq(libinput_device_config_accel_get_default_speed(device),
+			    0.0);
+	ck_assert_double_eq(libinput_device_config_accel_get_speed(device),
+			    0.0);
 }
 END_TEST
 
@@ -989,7 +998,6 @@ START_TEST(pointer_accel_direction_change)
 	struct libinput_event_pointer *pev;
 	int i;
 	double delta;
-	double max_accel;
 
 	litest_drain_events(li);
 
@@ -1009,16 +1017,14 @@ START_TEST(pointer_accel_direction_change)
 		pev = libinput_event_get_pointer_event(event);
 
 		delta = libinput_event_pointer_get_dx(pev);
-		ck_assert(delta <= 0.0);
-		max_accel = delta;
+		ck_assert_double_le(delta, 0.0);
 		libinput_event_destroy(event);
 		event = libinput_get_event(li);
 	} while (libinput_next_event_type(li) != LIBINPUT_EVENT_NONE);
 
 	pev = libinput_event_get_pointer_event(event);
 	delta = libinput_event_pointer_get_dx(pev);
-	ck_assert(delta > 0.0);
-	ck_assert(delta < -max_accel);
+	ck_assert_double_gt(delta, 0.0);
 	libinput_event_destroy(event);
 }
 END_TEST
@@ -1413,6 +1419,32 @@ START_TEST(middlebutton_default_disabled)
 }
 END_TEST
 
+START_TEST(pointer_time_usec)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+	struct libinput_event_pointer *ptrev;
+	struct libinput_event *event;
+
+	litest_drain_events(dev->libinput);
+
+	litest_event(dev, EV_REL, REL_X, 1);
+	litest_event(dev, EV_REL, REL_Y, 1);
+	litest_event(dev, EV_SYN, SYN_REPORT, 0);
+
+	litest_wait_for_event(li);
+
+	event = libinput_get_event(li);
+	ptrev = litest_is_motion_event(event);
+
+	ck_assert_int_eq(libinput_event_pointer_get_time(ptrev),
+			 libinput_event_pointer_get_time_usec(ptrev) / 1000);
+
+	libinput_event_destroy(event);
+	litest_drain_events(dev->libinput);
+}
+END_TEST
+
 void
 litest_setup_tests(void)
 {
@@ -1460,4 +1492,6 @@ litest_setup_tests(void)
 	litest_add_for_device("pointer:middlebutton", middlebutton_default_alps, LITEST_ALPS_SEMI_MT);
 
 	litest_add_ranged("pointer:state", pointer_absolute_initial_state, LITEST_ABSOLUTE, LITEST_ANY, &axis_range);
+
+	litest_add("pointer:time", pointer_time_usec, LITEST_RELATIVE, LITEST_ANY);
 }
