@@ -35,15 +35,6 @@
 
 #include "shared.h"
 
-static inline const char*
-bool_to_str(bool b)
-{
-	if (b)
-		return "yes";
-	else
-		return "no";
-}
-
 static const char *
 tap_default(struct libinput_device *device)
 {
@@ -181,6 +172,18 @@ click_defaults(struct libinput_device *device)
 	return str;
 }
 
+static const char *
+dwt_default(struct libinput_device *device)
+{
+	if (!libinput_device_config_dwt_is_available(device))
+		return "n/a";
+
+	if (libinput_device_config_dwt_get_default_enabled(device))
+		return "enabled";
+	else
+		return "disabled";
+}
+
 static void
 print_device_notify(struct libinput_event *ev)
 {
@@ -244,6 +247,8 @@ print_device_notify(struct libinput_event *ev)
 	printf("Click methods:    %s\n", str);
 	free(str);
 
+	printf("Disable-w-typing: %s\n", dwt_default(dev));
+
 	printf("\n");
 }
 
